/* NETWORK INTERFACING 
	- prebuilt functions, 1.0

	available functions: 
	- loadProfile
		get profile object
	- loadPosts 
		get a list of a user's posts
	- loadPostContent
		get content of each post and append

	- loadUsersCentral
		get a list of users from an external source
	- usersProfiles
		use a list of users and return their profile
	- userAndTheirPosts
		use a list of users and return their profile + all their posts

	- isOwner
		check if you are the site's owner
	- writePost
		submit a post interface
	
*/

/******************************************************************************/

async function loadProfile(archive) {
  var profile = await archive.readFile("/profile.json");
  profile = JSON.parse(profile);

  var userInfo = {
    profile: profile,
    archive: archive
  };

  return userInfo;
}

/******************************************************************************/

async function loadPosts(archive) {
  var posts = await archive.readdir("/posts/", { stat: true });

  var userPosts = {
    posts: posts,
    archive: archive
  };

  return userPosts;
}

/******************************************************************************/

async function loadPostContent(archive, post) {
  var postLink = "/posts/" + post.name;
  // console.log(postLink)
  var postContent = await archive.readFile(postLink);

  var postAndArchive = {
    post: JSON.parse(postContent),
    archive: archive
  };

  return postAndArchive;
}

/******************************************************************************/

async function loadUsersCentral(archive, fileName) {
  var getUserList = await archive.readFile(fileName);
  getUserList = JSON.parse(getUserList);

  var userList = {
    users: getUserList.users,
    archive: archive
  };

  return userList;
}

/******************************************************************************/

async function isOwner(archive) {
  var pageInfo = await archive.getInfo();
  return pageInfo.isOwner;
}

/******************************************************************************/

function loadTags(tagsData) {
  let tagClass = tagsData == undefined ? "hidden tags" : "tags";
  let tags = tagsData == undefined ? "no tags" : tagsData;
  return `<p class=${tagClass}>${tags} •</p>`;
}

/* */

function writePost(archive, postSubmission) {
  var archive = archive;

  // when someone clicks submit:
  postSubmission.addEventListener("submit", function(e) {
    e.preventDefault(); // avoid default behavior
    var formRecieved = e.target,
      formTitle = formRecieved.elements["Title"].value.toString(),
      formContents = formRecieved.elements["Content"].value.toString(),
      formTags = formRecieved.elements["Tags"].value.toString(),
      timestamp = new Date().getTime();

    // set up object to submit to post:
    var postContent = {
      title: formTitle,
      timestamp: timestamp,
      tags: formTags,
      content: formContents
    };

    // use archive (the DatArchive) to write a file
    async function postFile(archive, postContent) {
      // console.log(postContent),
      await archive.writeFile(
        "/posts/post-" + postContent.timestamp + ".json",
        JSON.stringify(postContent)
      );
    }

    postFile(archive, postContent)
      .then(function(event) {
        console.log("post is posted!");
      })
      .catch(function(error) {
        console.log("error\n", error);
      });
  });
}

/******************************************************************************/
/* RECURSIVE FUNCTIONS BELOW */
/* https://en.wikipedia.org/wiki/Recursion_(computer_science) */
/******************************************************************************/

function usersProfiles(userCounter, userList, profilesContainer) {
  // recursive function listing all users
  var userUrl = new DatArchive(userList[userCounter]),
    userAmount = userList.length;

  loadProfile(userUrl)
    .then(function(userInfo) {
      profilesContainer.insertAdjacentHTML(
        "beforeend",
        `
			<li>
				<hr />
				<p>
					<a href="${userInfo.archive.url}" target="_blank">
						${userInfo.profile.username}
					</a>
				</p>
				<p>${userInfo.profile.bio}</p>
				<p>${userInfo.profile.email}</p>
			</li>
		`
      );

      if (userCounter < userAmount - 1) {
        //foreach appending posts for this user is over
        userCounter++;
        usersProfiles(userCounter, userList, profilesContainer); // move to next user
      }
    })
    .catch(function(error) {
      console.log("error thrown\n", error);
    });
}

/******************************************************************************/

function userAndTheirPosts(userCounter, userList, watchingContainer) {
  // get username from dat url
  var userUrl = new DatArchive(userList[userCounter]);
  userAmount = userList.length;

  let allPosts = [];

  // load user profile:
  loadProfile(userUrl)
    .then(function(userInfo) {
      var userId = "user-" + userInfo.archive.url.replace("dat://", ""); // dynamically generated id to be populated later

      watchingContainer.insertAdjacentHTML(
        "beforeend",
        `
			<li>
				<hr />
			
				<ul id="${userId}"></ul>
			</li>
		`
      );

      return userInfo.archive;
    })
    .then(function(userArchive) {
      // load posts of user:
      loadPosts(userArchive)
        .then(function(userPosts) {
          // this user's posts:
          console.log(userPosts);
          var userId = "user-" + userPosts.archive.url.replace("dat://", ""); // dynamically generated id

          var amountOfPosts = userPosts.posts.length,
            postCounter = 0;

          userPosts.posts.forEach(function(post) {
            loadPostContent(userUrl, post).then(function(postAndArchive) {
              var post = postAndArchive.post;
              var thisPostContent = post.content;
              allPosts.push(post);

              allPosts.sort(function(x, y) {
                return x.timestamp - y.timestamp;
              });

              // rough image replacement:
              var userPostContainer = document.getElementById(userId); // dynamically generated id
              if (JSON.stringify(post.content).includes('src=\\"')) {
                thisPostContent = JSON.stringify(thisPostContent).replace(
                  'src=\\"',
                  'src="' + postAndArchive.archive.url + "/"
                );
              } else if (JSON.stringify(post.content).includes("src=\\'")) {
                thisPostContent = JSON.stringify(thisPostContent).replace(
                  "src=\\'",
                  "src='" + postAndArchive.archive.url + "/"
                );
              }

              var date = new Date(post.timestamp * 1000);
              // Hours part from the timestamp
              var hours = date.getHours();
              // Minutes part from the timestamp
              var minutes = "0" + date.getMinutes();
              // Seconds part from the timestamp
              var seconds = "0" + date.getSeconds();

              // Will display time in 10:30:23 format
              var formattedTime =
                hours + ":" + minutes.substr(-2) + ":" + seconds.substr(-2);

              let tagsDiv = loadTags(post.tags);

              userPostContainer.insertAdjacentHTML(
                "beforeend",
                `
              	<li>
              		<hr />
              		<p class="post-title">${post.title}</p>
                  <p>${thisPostContent}</p>
                  <div class="metadata">
                  ${tagsDiv}
                  <p class="post-time">${formattedTime}</p>
                  </div>

              	</li>
              `
              );

              postCounter++;
              if (
                postCounter >= amountOfPosts &&
                userCounter < userAmount - 1
              ) {
                //foreach appending posts for this user is over
                userCounter++;
                userAndTheirPosts(userCounter, userList, watchingContainer); // move to next user
              }
            });
          });

          // allPosts.map(function(post) {
          //   console.log("hello");
          //   console.log(post);
          // });
        })
        .catch(function(error) {
          console.log("error thrown\n", error);
        }); //end of loadPosts.then
    })
    .catch(function(error) {
      console.log("error thrown\n", error);
    });
} // end of function
